@@ -31,6 +31,7 @@ from trytond.pyson import PYSONDecoder, Eval, If
 from trytond.report import Report
 from trytond.sendmail import sendmail_transactional, SMTPDataManager
 from trytond.tools import grouped_slice, reduce_ids
+from trytond.tools.email_ import set_from_header
 from trytond.transaction import Transaction
 from trytond.url import http_host
 from trytond.wsgi import Base64Converter
@@ -111,13 +112,10 @@ class Scenario(Workflow, ModelSQL, ModelView):
     def get_models(cls):
         pool = Pool()
         Model = pool.get('ir.model')
-
-        models = [name for name, klass in pool.iterobject()
-            if issubclass(klass, MarketingAutomationMixin)]
-        models = Model.search([
-                ('model', 'in', models),
-                ])
-        return [(m.model, m.name) for m in models]
+        get_name = Model.get_name
+        models = (name for name, klass in pool.iterobject()
+            if issubclass(klass, MarketingAutomationMixin))
+        return [(m, get_name(m)) for m in models]
 
     @classmethod
     def get_record_count(cls, scenarios, names):
@@ -228,7 +226,7 @@ class Scenario(Workflow, ModelSQL, ModelView):
                         where=record.scenario == scenario.id)))
 
             records = []
-            for id_, in cursor.fetchall():
+            for id_, in cursor:
                 records.append(
                     Record(scenario=scenario, record=Model(id_)))
 
@@ -302,7 +300,7 @@ class Activity(ModelSQL, ModelView):
             ], "Action")
 
     # Send E-mail
-    email_from = fields.Char("From",
+    email_from = fields.Char("From", translate=True,
         states={
             'invisible': Eval('action') != 'send_email',
             },
@@ -365,7 +363,7 @@ class Activity(ModelSQL, ModelView):
 
     @classmethod
     def view_attributes(cls):
-        return [
+        return super().view_attributes() + [
             ('//group[@id="email"]', 'states', {
                     'invisible': Eval('action') != 'send_email',
                     }),
@@ -589,19 +587,20 @@ class Activity(ModelSQL, ModelView):
                     yield END, QName('img'), pos
                 yield kind, data, pos
 
+        context = self.email_context(record)
         title = (TextTemplate(translated.email_title)
-            .generate(record=record.record)
+            .generate(**context)
             .render())
         template = MarkupTemplate(translated.email_template)
         content = (template
-            .generate(record=record.record)
+            .generate(**context)
             .filter(convert_href)
             .render())
 
-        msg = MIMEMultipart('alternative')
-        msg['From'] = (self.email_from
-            or config.get('marketing', 'email_from')
+        from_ = (config.get('marketing', 'email_from')
             or config.get('email', 'from'))
+        msg = MIMEMultipart('alternative')
+        set_from_header(msg, from_, translated.email_from or from_)
         msg['To'] = to
         msg['Subject'] = Header(title, 'utf-8')
         if html2text:
@@ -615,7 +614,18 @@ class Activity(ModelSQL, ModelView):
         to_addrs = [a for _, a in getaddresses([to])]
         if to_addrs:
             sendmail_transactional(
-                self.email_from, to_addrs, msg, datamanager=smtpd_datamanager)
+                from_, to_addrs, msg, datamanager=smtpd_datamanager)
+
+    def email_context(self, record):
+        return {
+            'record': record.record,
+            'format_date': Report.format_date,
+            'format_datetime': Report.format_datetime,
+            'format_timedelta': Report.format_timedelta,
+            'format_currency': Report.format_currency,
+            'format_number': Report.format_number,
+            'datetime': datetime,
+            }
 
 
 class Record(ModelSQL, ModelView):
@@ -659,10 +669,8 @@ class Record(ModelSQL, ModelView):
         if not self.scenario:
             return Scenario.get_models()
 
-        model, = Model.search([
-                ('model', '=', self.scenario.model),
-                ])
-        return [(model.model, model.name)]
+        model = self.scenario.model
+        return [(model, Model.get_name(model))]
 
     def eval(self, expression):
         env = {}
